#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Dremio connector to use most of the Dremio-API functionality in python.
All requests and responses are typed as dataclasses. For further information read our README or visit the repo on github.
"""
__author__ = "Holger Zernetsch, Jan Pietsch"
__copyright__ = "Copyright 2024, Continental AG"
__version__ = "0.3.1"
